import { __awaiter } from "tslib";
import { logger, MessageType, dataServerResultFilter, } from '@genesislcap/foundation-comms';
import { DOM } from '@microsoft/fast-element';
import { BaseServerSideDatasource } from './server-side.resource-base';
/**
 * The DATASERVER IServerSideDatasource implementation, used for SSRM (Server-Side Row Model) in the grid.
 * @alpha
 */
export class DataserverServerSideDatasource extends BaseServerSideDatasource {
    constructor(options) {
        super(options);
        this.createDataserverStreamFunc = options.createDataserverStreamFunc;
    }
    refreshDatasource(params) {
        const _super = Object.create(null, {
            refreshDatasource: { get: () => super.refreshDatasource }
        });
        return __awaiter(this, void 0, void 0, function* () {
            this.dataserverStreamSubscription.unsubscribe();
            this.dataserverStreamSubscription = undefined;
            yield _super.refreshDatasource.call(this, params);
        });
    }
    getRows(params) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setupFilteringAndSorting(params);
            if (this.pagination && !this.isNewPageSize && this.currentSequenceId > 0) {
                params.success(this.lastSuccessRowData);
                return;
            }
            if (!this.dataserverStream) {
                this.dataserverStream = yield this.createDataserverStreamFunc(this.resourceParams);
            }
            if (this.currentSequenceId >= 1 &&
                (this.moreRows ||
                    params.request.startRow >= Number(this.maxRows) ||
                    params.request.startRow === 0)) {
                if (this.pagination) {
                    this.resourceParams.VIEW_NUMBER = this.getPageNumber(params.request.startRow, this.maxRows, this.maxView, this.zeroBasedViewNumber);
                    // if (this.resourceParams.VIEW_NUMBER === 1) {
                    //   await this.refreshDatasource(params);
                    //   return;
                    // }
                }
                this.connect.getMoreRows(this.sourceRef, this.resourceParams.VIEW_NUMBER);
            }
            let applyResult = true;
            this.dataserverStreamSubscription = this.dataserverStream.subscribe((dataserverResult) => {
                if (applyResult) {
                    DOM.queueUpdate(() => __awaiter(this, void 0, void 0, function* () {
                        if (this.resourceParams.CRITERIA_MATCH) {
                            const updatedInfo = yield this.connect.snapshot(this.resourceName, {
                                MAX_ROWS: this.maxView,
                                MAX_VIEW: this.maxView,
                                CRITERIA_MATCH: this.resourceParams.CRITERIA_MATCH,
                            });
                            if (!updatedInfo.ROW && !updatedInfo.ROWS_COUNT) {
                                logger.warn('No data returned when criteria is applied.');
                                this.serverRowsCount = 0;
                                this.clientRowsCount = 0;
                            }
                            else {
                                this.serverRowsCount = this.resourceParams.CRITERIA_MATCH
                                    ? updatedInfo.ROW.length
                                    : updatedInfo.ROWS_COUNT;
                            }
                        }
                        this.applyServerSideData(params, dataserverResult);
                        applyResult = false;
                    }));
                }
            });
        });
    }
    applyServerSideData(params, result) {
        var _a;
        const messageType = result.MESSAGE_TYPE;
        if (messageType &&
            (messageType === MessageType.LOGOFF_ACK || messageType === MessageType.MSG_NACK)) {
            params.fail();
            return;
        }
        this.moreRows = result.MORE_ROWS;
        if (result.ROW) {
            const nextMessage = dataServerResultFilter(result);
            this.handleCurrentStreamLoad(nextMessage);
        }
        else {
            this.rowData = new Map();
            const successRowData = { rowData: [] };
            successRowData.rowCount = 0;
            params.success(successRowData);
            return;
        }
        this.currentSequenceId = result.SEQUENCE_ID;
        if (this.currentSequenceId === 1) {
            this.sourceRef = result.SOURCE_REF;
            if (this.serverRowsCount === 0) {
                this.serverRowsCount = (_a = result.ROWS_COUNT) !== null && _a !== void 0 ? _a : this.rowData.size;
            }
        }
        const successRowData = {
            rowData: Array.from(this.rowData.values()),
        };
        successRowData.rowCount = this.getCorrectRowCount(params);
        this.lastSuccessRowData = successRowData;
        params.success(successRowData);
    }
    destroy() {
        const _super = Object.create(null, {
            destroy: { get: () => super.destroy }
        });
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.dataserverStream = undefined;
            (_a = this.dataserverStreamSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            this.dataserverStreamSubscription = undefined;
            yield _super.destroy.call(this);
        });
    }
    handleCurrentStreamLoad(result) {
        var _a, _b;
        if (!result)
            return;
        const rows = new Map();
        (_a = result.inserts) === null || _a === void 0 ? void 0 : _a.forEach((insertData) => {
            rows.set(insertData[this.rowId], insertData);
        });
        this.clientRowsCount += (_b = result.inserts.length) !== null && _b !== void 0 ? _b : 0;
        this.rowData = rows;
    }
}
